#ifndef _fma_flags_h_
#define _fma_flags_h_

/*
 * Run-mode flags for FMAs.  This is broken out since there is no telling
 * who might need to include it.
 */

/*
 * FMA flags
 */
#define FMA_FLAG_CAN_ROUTE   0x00000001	/* can compute routes from a map */
#define FMA_FLAG_CAN_VERIFY  0x00000002	/* can patricipate in fabric verify */
#define FMA_FLAG_CAN_PROXY   0x00000004	/* can be a proxy to FMS */
#define FMA_FLAG_NEEDS_PROXY 0x00000008	/* needs to proxy to talk to FMS */
#define FMA_FLAG_HAS_FMS     0x00000010	/* is in contact with FMS */
#define FMA_FLAG_CAN_DO_LAG  0x00000020	/* can perform ethernet link aggr. */
#define FMA_FLAG_IS_EMBEDDED 0x00000040	/* This is a processor on a linecard */
#define FMA_FLAG_CAN_DISTRIBUTE 0x00000080 /* capable of distributing maps */
#define FMA_FLAG_IS_GATEWAY  0x00000100	/* is an ethernet gateway */
#define FMA_FLAG_CAN_DO_MOE  0x00000200 /* can be an MoE peer */

/* Some 2Z code accidentally got released with this value */
#define FMA_FLAG_OBSOLETE_IS_EMBEDDED 0x80000000

#define FMA_FLAGS_FMS_PRIVATE 0x70000000 /* FMS gets a few bits for itself */

 
/* some flag values do not go into the database */
#define FMA_FLAGS_NO_DB_MASK (~(FMA_FLAG_HAS_FMS))

/*
 * Mapper levels can tell us about other guy's FMS connectivity
 */
#define FMA_MAX_MAPPER_LEVEL 50		/* MAX settable mapper level */
#define FMA_MOM_LEVEL_BONUS 51		/* MoM nodes get a bonus */
#define FMA_PROXY_MAPPER_LEVEL (FMA_MOM_LEVEL_BONUS+FMA_MAX_MAPPER_LEVEL+1)
#define FMA_FMS_MAPPER_LEVEL (FMA_PROXY_MAPPER_LEVEL+1)

static inline int
lf_flag_fixup(
  int old_flags)
{
  /* If someone is using obsolete embedded flag, convert to correct value */
  if (old_flags & FMA_FLAG_OBSOLETE_IS_EMBEDDED) {
    return (old_flags & ~FMA_FLAG_OBSOLETE_IS_EMBEDDED) | FMA_FLAG_IS_EMBEDDED;
  }

  return old_flags;
}

#endif /* _fma_flags_h_ */
