/*
 * Utility functions
 */
#include <time.h>
#include <string.h>
#include <stdio.h>

#include "libfma.h"
#include "lf_internal.h"

/*
 * Return a timestamp string.  This is a static string.
 */
char *
lf_timestamp()
{
  time_t now;
  static lf_string_t str;

  time(&now);
  strcpy(str, asctime(localtime(&now)));
  str[strlen(str)-1] = '\0';
  return str;
}

/*
 * Convert a string to a MAC address
 */
void
lf_string_to_mac(
  lf_mac_addr_t mac_addr,
  char *mac_str)
{
  int int_mac[6];
  int i;

  sscanf(mac_str, LF_MAC_FORMAT,
      &int_mac[0],
      &int_mac[1],
      &int_mac[2],
      &int_mac[3],
      &int_mac[4],
      &int_mac[5]);

  for (i=0; i<6; ++i) {
    mac_addr[i] = int_mac[i];
  }
}

/*
 * Hex dump
 */
void
lf_hex_dump(
  FILE *fp,
  void *vbp,
  int len)
{
  unsigned char *bp;
  int i;

  bp = vbp;
  for (i=0; i<len; ++i) {
    if (i%16 == 0) fprintf(fp, "%08x:", i);
    if (i%16 == 8) fprintf(fp, " ");
    fprintf(fp, " %02x", *bp++);
    if (i%16 == 15) fprintf(fp, "\n");
  }
  if (i%16 != 0) fprintf(fp, "\n");
}

/*
 * Cycle log files to save old ones
 */
void
lf_cycle_logs(
  char *logname,
  int num_logs)
{
  lf_string_t from;
  lf_string_t to;
  int n;

  sprintf(to, "%s.%d", logname, num_logs);
  for (n=num_logs-1; n>0; --n) {
    sprintf(from, "%s.%d", logname, n);
    (void) rename(from, to);
    strcpy(to, from);
  }
  (void) rename(logname, to);
}
